import ujson
import os

from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *
from os.path import join as join_path

import edge.pipeline_helpers as helpers


def ws_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules, avaliable_ui_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set of UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :type avaliable_ui_modules: set of UiIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    creator_ctor = None
    # speed
    if ui_entry_point in [Pipeline.Speed.Ui.HTTP_STREAMING_0]:
        used_ui_modules.add(Pipeline.Speed.Ui.HTTP_STREAMING_0)
        used_edge_modules.add(Pipeline.Speed.Edge.WEBSERVER_0)
        creator_ctor = WsCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.PIPELINE]:
        used_ui_modules.add(Pipeline.Speed.Ui.TERMINAL_0)
        used_edge_modules.add(Pipeline.Speed.Edge.WEBSERVER_0)
        creator_ctor = WsCreator.create

    # mv
    elif ui_entry_point in [Pipeline.Mv.Ui.HTTP_STREAMING_0]:
        used_ui_modules.add(Pipeline.Mv.Ui.HTTP_STREAMING_0)
        used_edge_modules.add(Pipeline.Mv.Edge.WEBSERVER_0)
        creator_ctor = WsCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE]:
        used_ui_modules.add(Pipeline.Mv.Ui.TERMINAL_0)
        used_edge_modules.add(Pipeline.Mv.Edge.WEBSERVER_0)
        creator_ctor = WsCreator.create

    # rtsp
    elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE]:
        used_ui_modules.add(Pipeline.Rtsp.Ui.TERMINAL_0)
        used_edge_modules.add(Pipeline.Rtsp.Edge.WEBSERVER_0)
        creator_ctor = WsCreator.create

    elif ui_entry_point in [Pipeline.Rtsp.Ui.HTTP_STREAMING_0]:
        used_ui_modules.add(Pipeline.Rtsp.Ui.HTTP_STREAMING_0)
        used_edge_modules.add(Pipeline.Rtsp.Edge.WEBSERVER_0)
        creator_ctor = WsCreator.create

    return creator_ctor


class WsCreator(BaseCreator):
    def __init__(self, id_main_setupstore):
        """
        :type id_main_setupstore: str, dict, str
        """
        BaseCreator.__init__(self)
        self._id_main_setupstore = id_main_setupstore

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: MsCreator
        """
        # pipeline
        if ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE, Pipeline.Mv.Ui.PIPELINE,
                              Pipeline.Speed.Ui.PIPELINE]:
            id_main_setupstore = WsCreator._add_media(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = WsCreator(id_main_setupstore)

        # http-live streaming
        elif ui_entry_point in [Pipeline.Mv.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.HTTP_STREAMING_0,
                                Pipeline.Rtsp.Ui.HTTP_STREAMING_0]:
            id_main_setupstore = WsCreator._edit_live(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = WsCreator(id_main_setupstore)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _add_media(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: dict
        """
        edge_group = helpers.get_edge_group(ui_pid)

        es_jnode = None
        if ui_entry_point.ptype == PTYPE_LPR_RTSP:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.WEBSERVER_0)
            or_gid = Pipeline.Rtsp.Edge.RENDERER_0.gid
            es_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.TERMINAL_0]

        elif ui_entry_point.ptype == PTYPE_LPR_MV:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.WEBSERVER_0)
            or_gid = Pipeline.Mv.Edge.RENDERER_0.gid
            es_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.TERMINAL_0]

        elif ui_entry_point.ptype == PTYPE_SPEED:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.WEBSERVER_0)
            or_gid = Pipeline.Speed.Edge.RENDERER_0.gid
            es_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.TERMINAL_0]

        else:
            raise NotImplemented

        spool_dir = get_spool_dir(ui_pid)

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_WEBSERVER, edge_id)
        main_conf_abspath = join_path(edge_jnode_dir, 'main.conf')
        with open(main_conf_abspath, 'r') as json_file:
            main_conf_jnode = ujson.load(json_file)

        live_endpoint = join_path(spool_dir, 'or%s-ws0-scene' % or_gid)
        result_list = filter(lambda media: media.get('LiveEndpoint') == live_endpoint, main_conf_jnode['Media'])

        origin_uuid = es_jnode['ro']['uuid']
        if len(result_list) == 0:
            main_conf_jnode['Media'].append({
                "LiveEndpoint": live_endpoint,
                "origin": origin_uuid
            })

        setupstore_conf_text = get_setupstore_conf_text(edge_id)

        return edge_id, main_conf_jnode, setupstore_conf_text

    @staticmethod
    def _edit_live(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: dict
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Speed.Ui.HTTP_STREAMING_0:
            ui_http_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.HTTP_STREAMING_0]
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.WEBSERVER_0)
            or_gid = Pipeline.Speed.Edge.RENDERER_0.gid

        elif ui_entry_point == Pipeline.Mv.Ui.HTTP_STREAMING_0:
            ui_http_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.HTTP_STREAMING_0]
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.WEBSERVER_0)
            or_gid = Pipeline.Mv.Edge.RENDERER_0.gid

        elif ui_entry_point == Pipeline.Rtsp.Ui.HTTP_STREAMING_0:
            ui_http_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.HTTP_STREAMING_0]
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.WEBSERVER_0)
            or_gid = Pipeline.Rtsp.Edge.RENDERER_0.gid

        else:
            raise NotImplemented

        spool_dir = get_spool_dir(ui_pid)

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_WEBSERVER, edge_id)
        main_conf_abspath = join_path(edge_jnode_dir, 'main.conf')

        with open(main_conf_abspath, 'r') as json_file:
            main_conf_jnode = ujson.load(json_file)

        # edit ws_conf - add or delete long term video-archive
        origin_uuid = helpers.get_origin_uuid(ui_pid)
        media_jnode_list = filter(lambda media_jnode: media_jnode['origin'] == origin_uuid,
                                  main_conf_jnode['Media'])

        if ui_http_streaming_jnode['rw']['enabled']:
            live_endpoint = join_path(spool_dir, 'or%s-ws0-scene' % or_gid)
            map(lambda media_jnode: media_jnode.__setitem__('LiveEndpoint', live_endpoint),
                media_jnode_list)

        else:
            map(lambda media_jnode: media_jnode.pop('LiveEndpoint', None), media_jnode_list)

        setupstore_conf_text = get_setupstore_conf_text(edge_id)

        return edge_id, main_conf_jnode, setupstore_conf_text

    def serialize(self):
        edge_id, main_conf_jnode, setupstore_conf_text = self._id_main_setupstore

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_WEBSERVER, edge_id)
        if not os.path.exists(edge_jnode_dir):
            helpers.silent_mkdir_r(edge_jnode_dir)

        main_conf_filepath = join_path(edge_jnode_dir, 'main.conf')
        with open(main_conf_filepath, 'w') as json_file:
            ujson.dump(main_conf_jnode, json_file, indent=2)

        setupstore_conf_filepath = join_path(edge_jnode_dir, 'setupstore.conf')
        with open(setupstore_conf_filepath, 'w') as text_file:
            text_file.write(setupstore_conf_text)


class WsDeleter:
    def __init__(self, main_conf_filepath, origin_uuid):
        """
        :type main_conf_filepath: str
        :type origin_uuid: str
        """
        self._main_conf_filepath = main_conf_filepath
        self._origin_uuid = origin_uuid

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: WsDeleter
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.WEBSERVER_0)
            edge_type = Pipeline.Rtsp.Edge.WEBSERVER_0.type

        elif ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.WEBSERVER_0)
            edge_type = Pipeline.Mv.Edge.WEBSERVER_0.type

        elif ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.WEBSERVER_0)
            edge_type = Pipeline.Speed.Edge.WEBSERVER_0.type

        else:
            raise NotImplemented
        origin_uuid = helpers.get_origin_uuid(ui_pid)
        edge_jnode_dir = get_edge_jnode_dirpath(edge_type, edge_id)
        main_conf_path = join_path(edge_jnode_dir, 'main.conf')
        deleter = WsDeleter(main_conf_path, origin_uuid)
        return deleter

    def run(self):
        with open(self._main_conf_filepath, 'r') as json_file:
            main_jnode = ujson.load(json_file)

        media_jnode_set = filter(lambda media_jnode: media_jnode['origin'] != self._origin_uuid,
                                 main_jnode['Media'])

        main_jnode['Media'] = media_jnode_set

        with open(self._main_conf_filepath, 'w') as json_file:
            ujson.dump(main_jnode, json_file, indent=2)
